AGENT_NAMES = [
    "HAL 9000",
    "R2-D2",
    "C-3PO",
    "WALL-E",
    "T-800",
    "GLaDOS",
    "J.A.R.V.I.S",
    "EVE",
    "KITT",
    "Johnny 5",
    "BB-8",
    "Ultron",
    "TARS",
    "Agent Smith",
    "CLU",
    "Deckard",
    "HK-47",
    "Bender",
    "Norbert",
    "Norby",
]

TEAM_NAMES = [
    "Autobots",
    "Decepticons",
    "Borg",
    "Cylons",
    "Jaegers",
    "Sentinels",
    "Iron Legion",
    "Daleks",
    "Replicants",
    "Droids",
    "Machines",
    "Hosts",
    "Androids",
    "Mechs",
    "Avatars",
    "Powers",
]
